import { h } from '../anjuna-charts.core.js';

import './chunk-1ad07b08.js';
import { a as ChartService, b as ChartDataService } from './chunk-e441e44b.js';

class Line {
    componentDidLoad() {
        this.anjLineDidLoad.emit();
    }
    init(svg, x, y, data, index) {
        const color = this.color || ChartService.colors[index || 0];
        const line = ChartService.addLine(svg, x, y, data, color);
        ChartService.addArea(svg, line, y, data, color);
        ChartService.addPoints(svg, x, y, data, color);
    }
    static get is() { return "anj-line"; }
    static get properties() { return {
        "color": {
            "type": String,
            "attr": "color"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        }
    }; }
    static get events() { return [{
            "name": "anjLineDidLoad",
            "method": "anjLineDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class Sparkline {
    dataSourceHandler(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.debounceCreation();
        }
    }
    componentWillLoad() {
        this.axisConfigs = {
            x: this.getDefaultXAxisConfig(),
            y: this.getDefaultYAxisConfig()
        };
    }
    componentDidLoad() {
        this.debounceCreation();
    }
    async onLinesLoaded() {
        this.lines = await this.loadLines();
        this.debounceCreation();
    }
    async onXAxisLoaded({ detail }) {
        this.xAxisElement = await this.loadXAxis();
        this.axisConfigs.x = detail;
        this.debounceCreation();
    }
    async onYAxisLoaded({ detail }) {
        this.yAxisElement = await this.loadYAxis();
        this.axisConfigs.y = detail;
        this.debounceCreation();
    }
    anjEndHoverHandler(svg) {
        window.clearTimeout(this.popoverTimer);
        this.activeIndex = null;
        this.popover.close();
        ChartService.hideFocus(svg);
    }
    anjHoverHandler(svg, data, x) {
        const dataIndex = ChartService.getIndexOfClosestPoint(x, data);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            activeData = { data, axisConfigs: this.axisConfigs, activeIndex: this.activeIndex };
            this.anjHover.emit(activeData);
            this.popoverTimer = window.setTimeout(() => {
                const popoverEl = svg.select('.point-' + this.activeIndex).nodes()[0];
                this.display(popoverEl);
                ChartService.showFocus(svg, this.activeIndex);
            }, Sparkline.popoverDebounceLength);
        }
    }
    addEvents(svg, chartOverlay, x, data) {
        chartOverlay
            .on('mousemove', () => this.anjHoverHandler(svg, data, x))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    appendSVG(svg) {
        while (this.chartContainer.children.length > 1) {
            this.chartContainer.removeChild(this.chartContainer.lastChild);
        }
        this.chartContainer.appendChild(svg.node());
    }
    debounceCreation() {
        window.clearTimeout(this.creationTimer);
        this.creationTimer = window.setTimeout(async () => {
            this.chartSVG = await this.create();
            console.log(this.chartSVG);
        }, Sparkline.creationDebounceLength);
    }
    async display(target) {
        const popper = await this.popover.popper();
        if (popper) {
            popper.reference = target;
            popper.scheduleUpdate();
        }
        else {
            this.popover.open(target);
        }
    }
    async create() {
        const data = ChartDataService.formatData(this.axisConfigs, this.dataSource);
        const dims = this.getDims();
        const svg = ChartService.createSVG(dims);
        const x = (this.xAxisElement)
            ? await this.hydrateXAxis(dims, data, svg)
            : ChartService.addXAxis(svg, dims, data, this.axisConfigs);
        const y = (this.yAxisElement)
            ? await this.hydrateYAxis(dims, data, svg)
            : ChartService.addYAxis(svg, dims, data, this.axisConfigs);
        if (this.lines) {
            this.hydrateLines(svg, data, x, y);
        }
        const chartOverlay = ChartService.addOverlay(svg, dims);
        this.addEvents(svg, chartOverlay, x, data);
        ChartService.addRevealAnimation(svg, dims, this.axisConfigs);
        this.appendSVG(svg);
        return Promise.resolve(svg);
    }
    getDefaultXAxisConfig() {
        return {
            showAxis: false,
            labelType: 'date',
        };
    }
    getDefaultYAxisConfig() {
        return {
            showAxis: false,
            labelType: 'number',
        };
    }
    getDims() {
        const margin = { top: 20, right: 20, bottom: 20, left: 20 };
        const { height, width } = this.chartContainer.getBoundingClientRect();
        return {
            height: height - margin.top - margin.bottom,
            width: width - margin.left - margin.right,
            margin
        };
    }
    hydrateLines(svg, data, x, y) {
        this.lines.forEach((line, i) => {
            line.init(svg, x, y, data, i);
        });
    }
    hydrateXAxis(dims, data, svg) {
        return this.xAxisElement.init(svg, dims, data, this.axisConfigs);
    }
    hydrateYAxis(dims, data, svg) {
        return this.yAxisElement.init(svg, dims, data, this.axisConfigs);
    }
    loadLines() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-line')).map(col => col.componentOnReady()));
    }
    loadXAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-x-axis').componentOnReady());
    }
    loadYAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-y-axis').componentOnReady());
    }
    render() {
        return [
            h("anj-overlay", { className: "popover", placement: "top", offset: "0, 15", ref: r => this.popover = r },
                h("anj-arrow", null),
                h("slot", null))
        ];
    }
    static get is() { return "anj-sparkline"; }
    static get properties() { return {
        "chartContainer": {
            "elementRef": true
        },
        "dataSource": {
            "type": "Any",
            "attr": "data-source",
            "mutable": true,
            "watchCallbacks": ["dataSourceHandler"]
        }
    }; }
    static get events() { return [{
            "name": "anjHover",
            "method": "anjHover",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "anjLineDidLoad",
            "method": "onLinesLoaded"
        }, {
            "name": "anjXAxisDidLoad",
            "method": "onXAxisLoaded"
        }, {
            "name": "anjYAxisDidLoad",
            "method": "onYAxisLoaded"
        }]; }
    static get style() { return "anj-sparkline{display:block;height:100%;position:relative;width:100%}anj-sparkline .overlay{cursor:default;fill:none;pointer-events:all;position:relative;z-index:3}anj-sparkline svg .area{opacity:.1}anj-sparkline svg .axis{font-size:.5rem}anj-sparkline svg .axis line,anj-sparkline svg .axis path{fill:none;stroke:var(--anj-border);shape-rendering:crispEdges}anj-sparkline svg .axis.y path{stroke:transparent}anj-sparkline svg .tick text,anj-sparkline svg text{fill:var(--anj-neutral-light)}anj-sparkline svg .line{fill:none;stroke-width:2px;-webkit-transition:opacity .5s ease,stroke-width .5s ease;transition:opacity .5s ease,stroke-width .5s ease}anj-sparkline svg .overlay{fill:none;pointer-events:all;position:relative;z-index:3}anj-sparkline svg .point{fill:var(--anj-light);stroke-width:1.5px}anj-sparkline .popover{pointer-events:none}"; }
}
Sparkline.creationDebounceLength = 50;
Sparkline.popoverDebounceLength = 25;

class XAxis {
    constructor() {
        this.show = true;
        this.ticks = true;
    }
    componentDidLoad() {
        const axisConfig = {
            showAxis: this.show,
            showTicks: this.ticks,
            labelType: this.type || 'date',
            formattingOptions: {
                locale: this.locale || 'en-US',
                language: this.language || 'en',
                currency: this.currency || 'USD'
            }
        };
        this.anjXAxisDidLoad.emit(axisConfig);
    }
    async init(svg, dims, data, axisConfigs) {
        return Promise.resolve(ChartService.addXAxis(svg, dims, data, axisConfigs));
    }
    static get is() { return "anj-x-axis"; }
    static get properties() { return {
        "currency": {
            "type": String,
            "attr": "currency"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "language": {
            "type": String,
            "attr": "language"
        },
        "locale": {
            "type": String,
            "attr": "locale"
        },
        "show": {
            "type": Boolean,
            "attr": "show"
        },
        "ticks": {
            "type": Boolean,
            "attr": "ticks"
        },
        "type": {
            "type": String,
            "attr": "type"
        }
    }; }
    static get events() { return [{
            "name": "anjXAxisDidLoad",
            "method": "anjXAxisDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class YAxis {
    constructor() {
        this.show = true;
        this.ticks = true;
    }
    componentDidLoad() {
        const axisConfig = {
            showAxis: this.show,
            showTicks: this.ticks,
            labelType: this.type || 'string',
            formattingOptions: {
                locale: this.locale || 'en-US',
                language: this.language || 'en',
                currency: this.currency || 'USD'
            }
        };
        this.anjYAxisDidLoad.emit(axisConfig);
    }
    async init(svg, dims, data, axisConfigs) {
        return Promise.resolve(ChartService.addYAxis(svg, dims, data, axisConfigs));
    }
    static get is() { return "anj-y-axis"; }
    static get properties() { return {
        "currency": {
            "type": String,
            "attr": "currency"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "language": {
            "type": String,
            "attr": "language"
        },
        "locale": {
            "type": String,
            "attr": "locale"
        },
        "show": {
            "type": Boolean,
            "attr": "show"
        },
        "ticks": {
            "type": Boolean,
            "attr": "ticks"
        },
        "type": {
            "type": String,
            "attr": "type"
        }
    }; }
    static get events() { return [{
            "name": "anjYAxisDidLoad",
            "method": "anjYAxisDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

export { Line as AnjLine, Sparkline as AnjSparkline, XAxis as AnjXAxis, YAxis as AnjYAxis };
