import { h } from '../anjuna-charts.core.js';

import './chunk-1ad07b08.js';
import { a as ChartService, b as ChartDataService } from './chunk-e441e44b.js';

class Delta {
    deltaHandler(newValue) {
        if (!newValue) {
            return;
        }
        if (newValue < 0) {
            this.deltaDisplay = (newValue * -1).toString();
            this.positive = false;
        }
        else {
            this.deltaDisplay = newValue.toString();
            this.positive = true;
        }
    }
    periodFormatter(period) {
        switch (period) {
            case 0:
                return 'MoM';
            case 1:
                return 'YoY';
            case 2:
                return 'WoW';
            case 3:
                return 'DoD';
            default:
                return '';
        }
    }
    hostData() {
        return {
            'class': {
                'anj-delta-negative': !this.positive,
                'anj-has-data': !!this.delta
            }
        };
    }
    render() {
        return (this.delta && this.deltaDisplay) ? (h("span", null,
            this.positive ? h("i", { class: "z-icon-arrow-up" }) : h("i", { class: "z-icon-arrow-down" }),
            this.deltaDisplay,
            "% ",
            this.periodFormatter(this.period))) : undefined;
    }
    static get is() { return "anj-delta"; }
    static get properties() { return {
        "delta": {
            "type": Number,
            "attr": "delta",
            "watchCallbacks": ["deltaHandler"]
        },
        "period": {
            "type": Number,
            "attr": "period"
        }
    }; }
    static get style() { return "anj-delta{display:none}anj-delta.anj-has-data{border-top:1px solid var(--anj-border);color:var(--anj-success);display:inline-block;margin-top:.75rem;padding-top:.5rem;white-space:nowrap}anj-delta.anj-delta-negative{color:var(--anj-danger)}"; }
}

class PopoverContentSingle {
    activeDataHandler(newValue) {
        const x = newValue.data[newValue.activeIndex].x;
        const y = newValue.data[newValue.activeIndex].y;
        this.period = ChartService.calculateInterval(newValue.data);
        this.xDisplayValue = ChartDataService.xFormatter(this.period, newValue.axisConfigs.x, x);
        this.yDisplayValue = ChartDataService.yFormatter(newValue.axisConfigs.y, y);
        this.delta = PopoverContentSingle.getDelta(newValue);
    }
    static getDelta(activeData) {
        if (activeData.activeIndex === 0) {
            return undefined;
        }
        const prevY = parseInt(activeData.data[activeData.activeIndex - 1].y, 10);
        const y = parseInt(activeData.data[activeData.activeIndex].y, 10);
        return Math.round((1 - (y / prevY)) * -100);
    }
    render() {
        return (this.activeData) ? [
            h("div", { class: "main" },
                h("div", null, this.xDisplayValue),
                h("div", { class: "value" }, this.yDisplayValue),
                h("anj-delta", { delta: this.delta, period: this.period }))
        ] : undefined;
    }
    static get is() { return "anj-popover-content-single"; }
    static get properties() { return {
        "activeData": {
            "type": "Any",
            "attr": "active-data",
            "watchCallbacks": ["activeDataHandler"]
        }
    }; }
    static get style() { return "anj-popover-content-single{color:var(--anj-neutral);display:block;line-height:1;margin:.5rem 1rem;text-align:center;white-space:nowrap}anj-popover-content-single .value{color:var(--anj-primary);display:block;font-size:1.5rem;padding:.5rem .5rem 0}"; }
}

export { Delta as AnjDelta, PopoverContentSingle as AnjPopoverContentSingle };
